@@ -100,14 +100,25 @@ xfsm_verbose_real (const char *func,
           gchar *oldlogfile = g_strdup_printf ("%s.last", logfile);
           if (oldlogfile)
             {
-              rename (logfile, oldlogfile);
+              if (rename (logfile, oldlogfile) != 0)
+                {
+                  g_warning ("unable to rename logfile");
+                }
               g_free (oldlogfile);
             }
         }
 
-      fp = fopen (logfile, "w");
-      g_free (logfile);
-      fprintf(fp, "log file opened\n");
+      if (logfile)
+        {
+          fp = fopen (logfile, "w");
+          g_free (logfile);
+          fprintf(fp, "log file opened\n");
+        }
+    }
+
+  if (fp == NULL)
+    {
+      return;
     }
 
   fprintf (fp, "TRACE[%s:%d] %s(): ", file, line, func);
