@@ -102,8 +102,6 @@ struct _XfsmLogoutDialog
 
   /* dialog buttons */
   GtkWidget        *button_cancel;
-  GtkWidget        *button_ok;
-  GtkWidget        *button_close;
 
   /* error label */
   GtkWidget        *error_label;
@@ -146,9 +144,11 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
   gboolean       can_restart;
   gboolean       can_suspend = FALSE;
   gboolean       can_hibernate = FALSE;
+  gboolean       can_hybrid_sleep = FALSE;
   gboolean       can_switch_user = FALSE;
   gboolean       auth_suspend = FALSE;
   gboolean       auth_hibernate = FALSE;
+  gboolean       auth_hybrid_sleep = FALSE;
   GError        *error = NULL;
   XfconfChannel *channel;
   GtkWidget     *image;
@@ -269,7 +269,7 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
   gtk_widget_set_sensitive (button, can_shutdown);
   gtk_widget_show (button);
 
-  /* new row for suspend/hibernate */
+  /* new row for suspend/hibernate/hybrid sleep */
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, BORDER);
   gtk_box_pack_start (GTK_BOX (button_vbox), hbox, FALSE, TRUE, 0);
 
@@ -337,6 +337,37 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
         }
     }
 
+  /**
+   * Hybrid Sleep
+   *
+   * Hide the button if UPower is not installed or system cannot suspend
+   **/
+  if (!upower_not_found
+      && xfconf_channel_get_bool (channel, "/shutdown/ShowHybridSleep", TRUE))
+    {
+      if (xfsm_shutdown_can_hybrid_sleep (dialog->shutdown, &can_hybrid_sleep, &auth_hybrid_sleep, &error))
+        {
+          if (can_hybrid_sleep)
+            {
+              button = xfsm_logout_dialog_button (_("_Hybrid Sleep"), "system-hibernate",
+                                                  "xfsm-hibernate", XFSM_SHUTDOWN_HYBRID_SLEEP,
+                                                  dialog);
+
+              gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
+              gtk_widget_set_sensitive (button, auth_hybrid_sleep);
+              gtk_widget_show (button);
+
+              gtk_widget_show (hbox);
+            }
+        }
+      else
+        {
+          g_printerr ("%s: Querying hybrid-sleep failed: %s\n\n",
+                      PACKAGE_NAME, ERROR_MSG (error));
+          g_clear_error (&error);
+        }
+    }
+
   /**
    * Switch User
    *
@@ -436,7 +467,6 @@ xfsm_logout_dialog_set_mode (XfsmLogoutDialog *dialog,
     gtk_widget_set_visible (dialog->box[i], i == mode);
 
   gtk_widget_set_visible (dialog->button_cancel, mode != MODE_SHOW_ERROR);
-  gtk_widget_set_visible (dialog->button_close, mode == MODE_SHOW_ERROR);
 }
 
 
@@ -638,17 +668,26 @@ xfsm_logout_dialog_screenshot_save (GdkPixbuf   *screenshot,
 
 
 
+static void
+xfsm_logout_dialog_grab_callback (GdkSeat   *seat,
+                                  GdkWindow *window,
+                                  gpointer   user_data)
+{
+  /* ensure window is mapped to avoid unsuccessful grabs */
+  if (!gdk_window_is_visible (window))
+    gdk_window_show (window);
+}
+
+
+
 static gint
 xfsm_logout_dialog_run (GtkDialog *dialog,
                         gboolean   grab_input)
 {
   GdkWindow *window;
   gint       ret;
-#if !GTK_CHECK_VERSION (3, 20, 0)
   GdkDevice *device;
-#else
   GdkSeat   *seat;
-#endif
 
   if (grab_input)
     {
@@ -656,34 +695,19 @@ xfsm_logout_dialog_run (GtkDialog *dialog,
 
       window = gtk_widget_get_window (GTK_WIDGET (dialog));
 
-#if !GTK_CHECK_VERSION (3, 20, 0)
-          device = gtk_get_current_event_device ();
+      device = gtk_get_current_event_device ();
+      seat = device != NULL
+             ? gdk_device_get_seat (device)
+             : gdk_display_get_default_seat (gtk_widget_get_display (GTK_WIDGET (dialog)));
 
-          if (gdk_device_grab (device,
-                               window,
-                               GDK_OWNERSHIP_APPLICATION,
-                               FALSE,
-                               GDK_KEY_PRESS_MASK,
-                               NULL,
-                               GDK_CURRENT_TIME) != GDK_GRAB_SUCCESS)
-            {
-              g_critical ("Failed to grab the keyboard for logout window");
-            }
-#else
-          seat = gdk_device_get_seat (gtk_get_current_event_device ());
-
-          if (gdk_seat_grab (seat,
-                             window,
-                             GDK_SEAT_CAPABILITY_KEYBOARD,
-                             FALSE,
-                             NULL,
-                             gtk_get_current_event (),
-                             NULL,
-                             NULL) != GDK_GRAB_SUCCESS)
-            {
-              g_critical ("Failed to grab the keyboard for logout window");
-            }
-#endif
+      if (gdk_seat_grab (seat, window,
+                         GDK_SEAT_CAPABILITY_KEYBOARD,
+                         FALSE, NULL, NULL,
+                         xfsm_logout_dialog_grab_callback,
+                         NULL) != GDK_GRAB_SUCCESS)
+        {
+          g_critical ("Failed to grab the keyboard for logout window");
+        }
 
 #ifdef GDK_WINDOWING_X11
       /* force input to the dialog */
@@ -697,13 +721,8 @@ xfsm_logout_dialog_run (GtkDialog *dialog,
 
   ret = gtk_dialog_run (dialog);
 
-#if !GTK_CHECK_VERSION (3, 20, 0)
   if (grab_input)
-    gdk_device_ungrab (device, GDK_CURRENT_TIME);
-#else
-    if (grab_input)
     gdk_seat_ungrab (seat);
-#endif
 
   return ret;
 }
@@ -713,11 +732,11 @@ xfsm_logout_dialog_run (GtkDialog *dialog,
 gboolean
 xfsm_logout_dialog (const gchar      *session_name,
                     XfsmShutdownType *return_type,
-                    gboolean         *return_save_session)
+                    gboolean         *return_save_session,
+                    gboolean          accessibility)
 {
   gint              result;
   GtkWidget        *hidden;
-  gboolean          a11y;
   GtkWidget        *dialog;
   GdkScreen        *screen;
   gint              monitor;
@@ -727,6 +746,9 @@ xfsm_logout_dialog (const gchar      *session_name,
   XfconfChannel    *channel = xfsm_open_config ();
   gboolean          autosave;
   XfsmShutdown     *shutdown;
+  GdkDevice        *device;
+  GdkSeat          *seat;
+  gint              grab_count = 0;
 
   g_return_val_if_fail (return_type != NULL, FALSE);
   g_return_val_if_fail (return_save_session != NULL, FALSE);
@@ -755,47 +777,37 @@ xfsm_logout_dialog (const gchar      *session_name,
       monitor = 0;
     }
 
-  /* check if accessibility is enabled */
   hidden = gtk_invisible_new_for_screen (screen);
   gtk_widget_show (hidden);
-  a11y = GTK_IS_ACCESSIBLE (gtk_widget_get_accessible (hidden));
 
-  if (G_LIKELY (!a11y))
+  /* check if accessibility is enabled */
+  if (G_LIKELY (!accessibility))
     {
       /* wait until we can grab the keyboard, we need this for
        * the dialog when running it */
       for (;;)
         {
-#if !GTK_CHECK_VERSION (3, 20, 0)
-          GdkDevice *device = gtk_get_current_event_device ();
+          device = gtk_get_current_event_device ();
+          seat = device != NULL
+                 ? gdk_device_get_seat (device)
+                 : gdk_display_get_default_seat (gtk_widget_get_display (hidden));
 
-          if (gdk_device_grab (device,
-                               gtk_widget_get_window (hidden),
-                               GDK_OWNERSHIP_APPLICATION,
-                               FALSE,
-                               GDK_KEY_PRESS_MASK,
-                               NULL,
-                               GDK_CURRENT_TIME) == GDK_GRAB_SUCCESS)
-            {
-              gdk_device_ungrab (device, GDK_CURRENT_TIME);
-              break;
-            }
-#else
-          GdkSeat *seat = gdk_device_get_seat (gtk_get_current_event_device ());
-
-          if (gdk_seat_grab (seat,
-                             gtk_widget_get_window (hidden),
+          if (gdk_seat_grab (seat, gtk_widget_get_window (hidden),
                              GDK_SEAT_CAPABILITY_KEYBOARD,
-                             FALSE,
-                             NULL,
-                             gtk_get_current_event (),
-                             NULL,
+                             FALSE, NULL, NULL,
+                             xfsm_logout_dialog_grab_callback,
                              NULL) == GDK_GRAB_SUCCESS)
             {
               gdk_seat_ungrab (seat);
               break;
             }
-#endif
+
+          if (grab_count++ >= 40)
+            {
+              g_critical ("Failed to grab the keyboard for logout window");
+              break;
+            }
+
           g_usleep (G_USEC_PER_SEC / 20);
         }
 
@@ -805,7 +817,6 @@ xfsm_logout_dialog (const gchar      *session_name,
 
       /* display fadeout */
       fadeout = xfsm_fadeout_new (gdk_screen_get_display (screen));
-      gdk_flush ();
 
       dialog = g_object_new (XFSM_TYPE_LOGOUT_DIALOG,
                              "type", GTK_WINDOW_POPUP,
@@ -820,7 +831,7 @@ xfsm_logout_dialog (const gchar      *session_name,
   else
     {
       dialog = g_object_new (XFSM_TYPE_LOGOUT_DIALOG,
-                             "decorated", !a11y,
+                             "decorated", !accessibility,
                              "screen", screen, NULL);
 
       gtk_window_set_keep_above (GTK_WINDOW (dialog), TRUE);
@@ -834,7 +845,7 @@ xfsm_logout_dialog (const gchar      *session_name,
   /* set mode */
   xfsm_logout_dialog_set_mode (xfsm_dialog, MODE_LOGOUT_BUTTONS);
 
-  result = xfsm_logout_dialog_run (GTK_DIALOG (dialog), !a11y);
+  result = xfsm_logout_dialog_run (GTK_DIALOG (dialog), !accessibility);
 
   gtk_widget_hide (dialog);
 
