@@ -37,10 +37,12 @@
 #define SYSTEMD_POWEROFF_ACTION         "PowerOff"
 #define SYSTEMD_SUSPEND_ACTION          "Suspend"
 #define SYSTEMD_HIBERNATE_ACTION        "Hibernate"
+#define SYSTEMD_HYBRID_SLEEP_ACTION     "HybridSleep"
 #define SYSTEMD_REBOOT_TEST             "org.freedesktop.login1.reboot"
 #define SYSTEMD_POWEROFF_TEST           "org.freedesktop.login1.power-off"
 #define SYSTEMD_SUSPEND_TEST            "org.freedesktop.login1.suspend"
 #define SYSTEMD_HIBERNATE_TEST          "org.freedesktop.login1.hibernate"
+#define SYSTEMD_HYBRID_SLEEP_TEST       "org.freedesktop.login1.hibernate"
 
 
 
@@ -95,14 +97,13 @@ xfsm_systemd_init (XfsmSystemd *systemd)
 static void
 xfsm_systemd_finalize (GObject *object)
 {
+#ifdef HAVE_POLKIT
   XfsmSystemd *systemd = XFSM_SYSTEMD (object);
 
-#ifdef HAVE_POLKIT
   g_object_unref (G_OBJECT (systemd->authority));
   g_object_unref (G_OBJECT (systemd->subject));
-#endif
-
   g_object_unref (G_OBJECT (systemd->screensaver));
+#endif
 
   (*G_OBJECT_CLASS (xfsm_systemd_parent_class)->finalize) (object);
 }
@@ -268,6 +269,20 @@ xfsm_systemd_try_hibernate (XfsmSystemd  *systemd,
 
 
 
+gboolean
+xfsm_systemd_try_hybrid_sleep (XfsmSystemd  *systemd,
+                               GError      **error)
+{
+  if (!xfsm_systemd_lock_screen (systemd, error))
+    return FALSE;
+
+  return xfsm_systemd_try_method (systemd,
+                                  SYSTEMD_HYBRID_SLEEP_ACTION,
+                                  error);
+}
+
+
+
 gboolean
 xfsm_systemd_can_restart (XfsmSystemd  *systemd,
                           gboolean     *can_restart,
@@ -327,3 +342,21 @@ xfsm_systemd_can_hibernate (XfsmSystemd  *systemd,
   *auth_hibernate = *can_hibernate;
   return ret;
 }
+
+
+
+gboolean
+xfsm_systemd_can_hybrid_sleep (XfsmSystemd  *systemd,
+                               gboolean     *can_hybrid_sleep,
+                               gboolean     *auth_hybrid_sleep,
+                               GError      **error)
+{
+  gboolean ret = FALSE;
+
+  ret = xfsm_systemd_can_method (systemd,
+                                 can_hybrid_sleep,
+                                 SYSTEMD_HYBRID_SLEEP_TEST,
+                                 error);
+  *auth_hybrid_sleep = *can_hybrid_sleep;
+  return ret;
+}
